import { Injectable } from '@angular/core';
import { NatsService } from 'app/core/nats/services/nats.service';
import * as nats from 'nats.ws';

import { Error } from 'app/modules/proto/protobuf/rpc_pb';

import {
  ListMemberSettlementSettingsRequest,
  ListMemberSettlementSettingsResponse,
  CreateMemberSettlementSettingRequest,
  CreateMemberSettlementSettingResponse,
  UpdateMemberSettlementSettingRequest,
  UpdateMemberSettlementSettingResponse,
  DeleteMemberSettlementSettingRequest,
  DeleteMemberSettlementSettingResponse,
  GetMemberSettlementSettingResponse,
  GetMemberSettlementSettingRequest,
} from 'app/modules/proto/c2se/member_settlement_setting_pb';
import {
  SUBJECT_LIST_MEMBER_SETTLEMENT_SETTINGS,
  SUBJECT_CREATE_MEMBER_SETTLEMENT_SETTING,
  SUBJECT_UPDATE_MEMBER_SETTLEMENT_SETTING,
  SUBJECT_DELETE_MEMBER_SETTLEMENT_SETTING,
  SUBJECT_GET_MEMBER_SETTLEMENT_SETTING,
} from 'app/modules/proto/c2se/backend/member_settlement_setting_pb';

@Injectable({
  providedIn: 'root',
})
export class MemberSettlementSettingService {
  /**
   * Constructor
   */
  constructor(private __natsService: NatsService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Accessors
  // -----------------------------------------------------------------------------------------------------

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  createMemberSettlementSetting(): Promise<CreateMemberSettlementSettingResponse.Result> {
    return new Promise<CreateMemberSettlementSettingResponse.Result>(
      (resolve, reject) => {
        let req = new CreateMemberSettlementSettingRequest();

        this.__natsService
          .request<CreateMemberSettlementSettingResponse.Result>(
            SUBJECT_CREATE_MEMBER_SETTLEMENT_SETTING,
            req.serializeBinary(),
            CreateMemberSettlementSettingResponse.deserializeBinary
          )
          .then((result) => {
            console.log('success', result, result.getMemberSettlementSetting());
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            return reject(e);
          });
      }
    );
  }

  listMemberSettlementSettings(): Promise<ListMemberSettlementSettingsResponse.Result> {
    return new Promise<ListMemberSettlementSettingsResponse.Result>(
      (resolve, reject) => {
        let req = new ListMemberSettlementSettingsRequest();

        this.__natsService
          .request<ListMemberSettlementSettingsResponse.Result>(
            SUBJECT_LIST_MEMBER_SETTLEMENT_SETTINGS,
            req.serializeBinary(),
            ListMemberSettlementSettingsResponse.deserializeBinary
          )
          .then((result) => {
            console.log(
              'success',
              result,
              result.getMemberSettlementSettingsList()
            );
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            reject(e);
          });
      }
    );
  }

  getMemberSettlementSetting(
    id: string
  ): Promise<GetMemberSettlementSettingResponse.Result> {
    return new Promise<GetMemberSettlementSettingResponse.Result>(
      (resolve, reject) => {
        let req = new GetMemberSettlementSettingRequest();
        req.setId(id);

        this.__natsService
          .request<GetMemberSettlementSettingResponse.Result>(
            SUBJECT_GET_MEMBER_SETTLEMENT_SETTING,
            req.serializeBinary(),
            GetMemberSettlementSettingResponse.deserializeBinary
          )
          .then((result) => {
            console.log('success', result, result.getMemberSettlementSetting());
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            return reject(e);
          });
      }
    );
  }

  updateMemberSettlementSetting(
    req: UpdateMemberSettlementSettingRequest
  ): Promise<UpdateMemberSettlementSettingResponse.Result> {
    return new Promise<UpdateMemberSettlementSettingResponse.Result>(
      (resolve, reject) => {
        this.__natsService
          .request<UpdateMemberSettlementSettingResponse.Result>(
            SUBJECT_UPDATE_MEMBER_SETTLEMENT_SETTING,
            req.serializeBinary(),
            UpdateMemberSettlementSettingResponse.deserializeBinary
          )
          .then((result) => {
            console.log('success', result, result.getMemberSettlementSetting());
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            return reject(e);
          });
      }
    );
  }

  deleteMemberSettlementSetting(): Promise<DeleteMemberSettlementSettingResponse.Result> {
    return new Promise<DeleteMemberSettlementSettingResponse.Result>(
      (resolve, reject) => {
        let req = new DeleteMemberSettlementSettingRequest();

        this.__natsService
          .request<DeleteMemberSettlementSettingResponse.Result>(
            SUBJECT_DELETE_MEMBER_SETTLEMENT_SETTING,
            req.serializeBinary(),
            DeleteMemberSettlementSettingResponse.deserializeBinary
          )
          .then((result) => {
            console.log('success', result);
            return resolve(result);
          })
          .catch((e: Error) => {
            console.log('failed', e);
            return reject(e);
          });
      }
    );
  }
}
